@@ -456,11 +456,16 @@ void mutt_alias_add_reverse (ALIAS *t)
   ADDRESS *ap;
   if (!t)
     return;
-  
+
+  /* Note that the address mailbox should be converted to intl form
+   * before using as a key in the hash.  This is currently done
+   * by all callers, but added here mostly as documentation.. */
+  mutt_addrlist_to_intl (t->addr, NULL);
+
   for (ap = t->addr; ap; ap = ap->next)
   {
     if (!ap->group && ap->mailbox)
-      hash_insert (ReverseAlias, ap->mailbox, ap, 1);
+      hash_insert (ReverseAlias, ap->mailbox, ap);
   }
 }
 
@@ -469,7 +474,11 @@ void mutt_alias_delete_reverse (ALIAS *t)
   ADDRESS *ap;
   if (!t)
     return;
-  
+
+  /* If the alias addresses were converted to local form, they won't
+   * match the hash entries. */
+  mutt_addrlist_to_intl (t->addr, NULL);
+
   for (ap = t->addr; ap; ap = ap->next)
   {
     if (!ap->group && ap->mailbox)
