@@ -330,7 +330,7 @@ void mutt_check_lookup_list (BODY *b, char *type, int len)
 
 /* returns -1 on error, 0 or the return code from mutt_do_pager() on success */
 int mutt_view_attachment (FILE *fp, BODY *a, int flag, HEADER *hdr,
-			  ATTACHPTR **idx, short idxlen)
+			  ATTACH_CONTEXT *actx)
 {
   char tempfile[_POSIX_PATH_MAX] = "";
   char pagerfile[_POSIX_PATH_MAX] = "";
@@ -571,8 +571,7 @@ int mutt_view_attachment (FILE *fp, BODY *a, int flag, HEADER *hdr,
     info.fp = fp;
     info.bdy = a;
     info.ctx = Context;
-    info.idx = idx;
-    info.idxlen = idxlen;
+    info.actx = actx;
     info.hdr = hdr;
 
     rc = mutt_do_pager (descrip, pagerfile,
@@ -1042,3 +1041,88 @@ int mutt_print_attachment (FILE *fp, BODY *a)
     return 0;
   }
 }
+
+void mutt_actx_add_attach (ATTACH_CONTEXT *actx, ATTACHPTR *attach)
+{
+  int i;
+
+  if (actx->idxlen == actx->idxmax)
+  {
+    actx->idxmax += 5;
+    safe_realloc (&actx->idx, sizeof (ATTACHPTR *) * actx->idxmax);
+    safe_realloc (&actx->v2r, sizeof (short) * actx->idxmax);
+    for (i = actx->idxlen; i < actx->idxmax; i++)
+      actx->idx[i] = NULL;
+  }
+
+  actx->idx[actx->idxlen++] = attach;
+}
+
+void mutt_actx_add_fp (ATTACH_CONTEXT *actx, FILE *new_fp)
+{
+  int i;
+
+  if (actx->fp_len == actx->fp_max)
+  {
+    actx->fp_max += 5;
+    safe_realloc (&actx->fp_idx, sizeof (FILE *) * actx->fp_max);
+    for (i = actx->fp_len; i < actx->fp_max; i++)
+      actx->fp_idx[i] = NULL;
+  }
+
+  actx->fp_idx[actx->fp_len++] = new_fp;
+}
+
+void mutt_actx_add_body (ATTACH_CONTEXT *actx, BODY *new_body)
+{
+  int i;
+
+  if (actx->body_len == actx->body_max)
+  {
+    actx->body_max += 5;
+    safe_realloc (&actx->body_idx, sizeof (BODY *) * actx->body_max);
+    for (i = actx->body_len; i < actx->body_max; i++)
+      actx->body_idx[i] = NULL;
+  }
+
+  actx->body_idx[actx->body_len++] = new_body;
+}
+
+void mutt_actx_free_entries (ATTACH_CONTEXT *actx)
+{
+  int i;
+
+  for (i = 0; i < actx->idxlen; i++)
+  {
+    if (actx->idx[i]->content)
+      actx->idx[i]->content->aptr = NULL;
+    FREE (&actx->idx[i]->tree);
+    FREE (&actx->idx[i]);
+  }
+  actx->idxlen = 0;
+  actx->vcount = 0;
+
+  for (i = 0; i < actx->fp_len; i++)
+    safe_fclose (&actx->fp_idx[i]);
+  actx->fp_len = 0;
+
+  for (i = 0; i < actx->body_len; i++)
+    mutt_free_body (&actx->body_idx[i]);
+  actx->body_len = 0;
+}
+
+void mutt_free_attach_context (ATTACH_CONTEXT **pactx)
+{
+  ATTACH_CONTEXT *actx;
+
+  if (!pactx || !*pactx)
+    return;
+
+  actx = *pactx;
+  mutt_actx_free_entries (actx);
+  FREE (&actx->idx);
+  FREE (&actx->v2r);
+  FREE (&actx->fp_idx);
+  FREE (&actx->body_idx);
+  FREE (pactx);  /* __FREE_CHECKED__ */
+}
