@@ -492,17 +492,20 @@ static int examine_mailboxes (MUTTMENU *menu, struct browser_state *state)
       tmp->msg_unread = Context->unread;
     }
 
+    strfcpy (buffer, NONULL (tmp->path), sizeof (buffer));
+    mutt_pretty_mailbox (buffer, sizeof (buffer));
+
 #ifdef USE_IMAP
     if (mx_is_imap (tmp->path))
     {
-      add_folder (menu, state, tmp->path, NULL, tmp);
+      add_folder (menu, state, buffer, NULL, tmp);
       continue;
     }
 #endif
 #ifdef USE_POP
     if (mx_is_pop (tmp->path))
     {
-      add_folder (menu, state, tmp->path, NULL, tmp);
+      add_folder (menu, state, buffer, NULL, tmp);
       continue;
     }
 #endif
@@ -528,9 +531,6 @@ static int examine_mailboxes (MUTTMENU *menu, struct browser_state *state)
 	s.st_mtime = st2.st_mtime;
     }
 
-    strfcpy (buffer, NONULL(tmp->path), sizeof (buffer));
-    mutt_pretty_mailbox (buffer, sizeof (buffer));
-
     add_folder (menu, state, buffer, &s, tmp);
   }
   while ((tmp = tmp->next));
@@ -611,7 +611,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
   char helpstr[LONG_STRING];
   char title[STRING];
   struct browser_state state;
-  MUTTMENU *menu;
+  MUTTMENU *menu = NULL;
   struct stat st;
   int i, killPrefix = 0;
   int multiple = (flags & MUTT_SEL_MULTI)  ? 1 : 0;
@@ -722,6 +722,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 
   menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_FOLDER,
     FolderHelp);
+  mutt_push_current_menu (menu);
 
   init_menu (&state, menu, title, sizeof (title), buffy);
 
@@ -908,7 +909,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	}
 
 	destroy_state (&state);
-	mutt_menuDestroy (&menu);
 	goto bail;
 
       case OP_BROWSER_TELL:
@@ -954,7 +954,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	  menu->current = 0; 
 	  menu->top = 0; 
 	  init_menu (&state, menu, title, sizeof (title), buffy);
-	  MAYBE_REDRAW (menu->redraw);
 	}
 	/* else leave error on screen */
 	break;
@@ -977,7 +976,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	    menu->current = 0;
 	    menu->top = 0;
 	    init_menu (&state, menu, title, sizeof (title), buffy);
-	    MAYBE_REDRAW (menu->redraw);
 	  }
 	}
 	break;
@@ -1015,7 +1013,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	      state.entrylen--;
 	      mutt_message _("Mailbox deleted.");
 	      init_menu (&state, menu, title, sizeof (title), buffy);
-	      MAYBE_REDRAW (menu->redraw);
 	    }
 	  }
 	  else
@@ -1080,7 +1077,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 		  mutt_error _("Error scanning directory.");
 		  if (examine_directory (menu, &state, LastDir, prefix) == -1)
 		  {
-		    mutt_menuDestroy (&menu);
 		    goto bail;
 		  }
 		}
@@ -1095,7 +1091,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	      mutt_perror (buf);
 	  }
 	}
-	MAYBE_REDRAW (menu->redraw);
 	break;
 	
       case OP_ENTER_MASK:
@@ -1151,7 +1146,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	    else
 	    {
 	      mutt_error _("Error scanning directory.");
-	      mutt_menuDestroy (&menu);
 	      goto bail;
 	    }
 	    killPrefix = 0;
@@ -1162,7 +1156,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	    }
 	  }
 	}
-	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_SORT:
@@ -1246,10 +1239,8 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	{
 	  strfcpy (f, buf, flen);
 	  destroy_state (&state);
-	  mutt_menuDestroy (&menu);
 	  goto bail;
 	}
-	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_BROWSER_VIEW_FILE:
@@ -1264,7 +1255,6 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	{
 	  strfcpy (f, state.entry[menu->current].name, flen);
 	  destroy_state (&state);
-	  mutt_menuDestroy (&menu);
 	  goto bail;
 	}
 	else
@@ -1285,7 +1275,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	  b = mutt_make_file_attach (buf);
 	  if (b != NULL)
 	  {
-	    mutt_view_attachment (NULL, b, MUTT_REGULAR, NULL, NULL, 0);
+	    mutt_view_attachment (NULL, b, MUTT_REGULAR, NULL, NULL);
 	    mutt_free_body (&b);
 	    menu->redraw = REDRAW_FULL;
 	  }
@@ -1296,7 +1286,13 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
   }
   
   bail:
-  
+
+  if (menu)
+  {
+    mutt_pop_current_menu (menu);
+    mutt_menuDestroy (&menu);
+  }
+
   if (!folder)
     strfcpy (LastDir, LastDirBackup, sizeof (LastDir));
   
