@@ -787,6 +787,8 @@ int crypt_get_keys (HEADER *msg, char **keylist, int oppenc_mode)
 {
   ADDRESS *adrlist = NULL, *last = NULL;
   const char *fqdn = mutt_fqdn (1);
+  char *self_encrypt = NULL;
+  size_t keylist_size;
 
   /* Do a quick check to make sure that we can find all of the encryption
    * keys if the user has requested this service.
@@ -819,6 +821,8 @@ int crypt_get_keys (HEADER *msg, char **keylist, int oppenc_mode)
            return (-1);
        }
        unset_option (OPTPGPCHECKTRUST);
+       if (option (OPTPGPSELFENCRYPT))
+         self_encrypt = PgpSelfEncryptAs;
      }
      if ((WithCrypto & APPLICATION_SMIME)
          && (msg->security & APPLICATION_SMIME))
@@ -828,9 +832,18 @@ int crypt_get_keys (HEADER *msg, char **keylist, int oppenc_mode)
            rfc822_free_address (&adrlist);
            return (-1);
        }
+       if (option (OPTSMIMESELFENCRYPT))
+         self_encrypt = SmimeSelfEncryptAs;
      }
   }
 
+  if (!oppenc_mode && self_encrypt && *self_encrypt)
+  {
+    keylist_size = mutt_strlen (*keylist);
+    safe_realloc (keylist, keylist_size + mutt_strlen (self_encrypt) + 2);
+    sprintf (*keylist + keylist_size, " %s", self_encrypt);  /* __SPRINTF_CHECKED__ */
+  }
+
   rfc822_free_address (&adrlist);
     
   return (0);
