@@ -479,7 +479,7 @@ static void resort_index (MUTTMENU *menu)
   if (menu->current < 0)
     menu->current = ci_first_message ();
 
-  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 }
 
 static const struct mapping_t IndexHelp[] = {
@@ -494,6 +494,59 @@ static const struct mapping_t IndexHelp[] = {
   { NULL,	 0 }
 };
 
+static void index_menu_redraw (MUTTMENU *menu)
+{
+  char buf[LONG_STRING];
+
+  if (menu->redraw & REDRAW_FULL)
+  {
+    menu_redraw_full (menu);
+    mutt_show_error ();
+  }
+
+#ifdef USE_SIDEBAR
+  if (menu->redraw & REDRAW_SIDEBAR)
+  {
+    mutt_sb_set_buffystats (Context);
+    menu_redraw_sidebar (menu);
+  }
+#endif
+
+  if (Context && Context->hdrs && !(menu->current >= Context->vcount))
+  {
+    menu_check_recenter (menu);
+
+    if (menu->redraw & REDRAW_INDEX)
+    {
+      menu_redraw_index (menu);
+      menu->redraw |= REDRAW_STATUS;
+    }
+    else if (menu->redraw & (REDRAW_MOTION_RESYNCH | REDRAW_MOTION))
+      menu_redraw_motion (menu);
+    else if (menu->redraw & REDRAW_CURRENT)
+      menu_redraw_current (menu);
+  }
+
+  if (menu->redraw & REDRAW_STATUS)
+  {
+    menu_status_line (buf, sizeof (buf), menu, NONULL (Status));
+    mutt_window_move (MuttStatusWindow, 0, 0);
+    SETCOLOR (MT_COLOR_STATUS);
+    mutt_paddstr (MuttStatusWindow->cols, buf);
+    NORMAL_COLOR;
+    menu->redraw &= ~REDRAW_STATUS;
+    if (option(OPTTSENABLED) && TSSupported)
+    {
+      menu_status_line (buf, sizeof (buf), menu, NONULL (TSStatusFormat));
+      mutt_ts_status(buf);
+      menu_status_line (buf, sizeof (buf), menu, NONULL (TSIconFormat));
+      mutt_ts_icon(buf);
+    }
+  }
+
+  menu->redraw = 0;
+}
+
 /* This function handles the message index window as well as commands returned
  * from the pager (MENU_PAGER).
  */
@@ -519,13 +572,19 @@ int mutt_index_menu (void)
   menu->color = index_color;
   menu->current = ci_first_message ();
   menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_MAIN, IndexHelp);
+  menu->custom_menu_redraw = index_menu_redraw;
+  mutt_push_current_menu (menu);
 
   if (!attach_msg)
     mutt_buffy_check(1); /* force the buffy check after we enter the folder */
 
   FOREVER
   {
-    tag = 0; /* clear the tag-prefix */
+    /* Clear the tag prefix unless we just started it.  Don't clear
+     * the prefix on a timeout (op==-2), but do clear on an abort (op==-1)
+     */
+    if (tag && op != OP_TAG_PREFIX && op != OP_TAG_PREFIX_COND && op != -2)
+      tag = 0;
 
     /* check if we need to resort the index because just about
      * any 'op' below could do mutt_enter_command(), either here or
@@ -609,68 +668,31 @@ int mutt_index_menu (void)
        do_buffy_notify = 1;
     }
 
-    if (op != -1)
+    if (op >= 0)
       mutt_curs_set (0);
 
-    if (menu->redraw & REDRAW_FULL)
-    {
-      menu_redraw_full (menu);
-      mutt_show_error ();
-    }
-
     if (menu->menu == MENU_MAIN)
     {
-#ifdef USE_SIDEBAR
-      if (menu->redraw & REDRAW_SIDEBAR || SidebarNeedsRedraw)
-      {
-        mutt_sb_set_buffystats (Context);
-        menu_redraw_sidebar (menu);
-      }
-#endif
-      if (Context && Context->hdrs && !(menu->current >= Context->vcount))
-      {
-	menu_check_recenter (menu);
+      index_menu_redraw (menu);
 
-	if (menu->redraw & REDRAW_INDEX)
-	{
-	  menu_redraw_index (menu);
-	  menu->redraw |= REDRAW_STATUS;
-	}
-	else if (menu->redraw & (REDRAW_MOTION_RESYNCH | REDRAW_MOTION))
-	  menu_redraw_motion (menu);
-	else if (menu->redraw & REDRAW_CURRENT)
-	  menu_redraw_current (menu);
+      /* give visual indication that the next command is a tag- command */
+      if (tag)
+      {
+        mutt_window_mvaddstr (MuttMessageWindow, 0, 0, "tag-");
+        mutt_window_clrtoeol (MuttMessageWindow);
       }
 
-      if (menu->redraw & REDRAW_STATUS)
-      {
-	menu_status_line (buf, sizeof (buf), menu, NONULL (Status));
-        mutt_window_move (MuttStatusWindow, 0, 0);
-	SETCOLOR (MT_COLOR_STATUS);
-	mutt_paddstr (MuttStatusWindow->cols, buf);
-	NORMAL_COLOR;
-	menu->redraw &= ~REDRAW_STATUS;
-	if (option(OPTTSENABLED) && TSSupported)
-	{
-	  menu_status_line (buf, sizeof (buf), menu, NONULL (TSStatusFormat));
-	  mutt_ts_status(buf);
-	  menu_status_line (buf, sizeof (buf), menu, NONULL (TSIconFormat));
-	  mutt_ts_icon(buf);
-	}
-      }
-
-      menu->redraw = 0;
       if (menu->current < menu->max)
-	menu->oldcurrent = menu->current;
+        menu->oldcurrent = menu->current;
       else
-	menu->oldcurrent = -1;
+        menu->oldcurrent = -1;
 
       if (option (OPTARROWCURSOR))
-	mutt_window_move (MuttIndexWindow, menu->current - menu->top + menu->offset, 2);
+        mutt_window_move (MuttIndexWindow, menu->current - menu->top + menu->offset, 2);
       else if (option (OPTBRAILLEFRIENDLY))
-	mutt_window_move (MuttIndexWindow, menu->current - menu->top + menu->offset, 0);
+        mutt_window_move (MuttIndexWindow, menu->current - menu->top + menu->offset, 0);
       else
-	mutt_window_move (MuttIndexWindow, menu->current - menu->top + menu->offset,
+        mutt_window_move (MuttIndexWindow, menu->current - menu->top + menu->offset,
                           MuttIndexWindow->cols - 1);
       mutt_refresh ();
 
@@ -679,8 +701,6 @@ int mutt_index_menu (void)
       {
 	mutt_flushinp ();
 	mutt_resize_screen ();
-	menu->redraw = REDRAW_FULL;
-	menu->menu = MENU_MAIN;
 	SigWinch = 0;
 	menu->top = 0; /* so we scroll the right amount */
 	/*
@@ -696,14 +716,27 @@ int mutt_index_menu (void)
 
       dprint(4, (debugfile, "mutt_index_menu[%d]: Got op %d\n", __LINE__, op));
 
-      if (op == -1)
-	continue; /* either user abort or timeout */
+      /* either user abort or timeout */
+      if (op < 0)
+      {
+        if (tag)
+          mutt_window_clearline (MuttMessageWindow, 0);
+        continue;
+      }
 
       mutt_curs_set (1);
 
       /* special handling for the tag-prefix function */
-      if (op == OP_TAG_PREFIX)
+      if (op == OP_TAG_PREFIX || op == OP_TAG_PREFIX_COND)
       {
+        /* A second tag-prefix command aborts */
+        if (tag)
+        {
+          tag = 0;
+          mutt_window_clearline (MuttMessageWindow, 0);
+          continue;
+        }
+
 	if (!Context)
 	{
 	  mutt_error _("No mailbox is open.");
@@ -712,55 +745,23 @@ int mutt_index_menu (void)
 
 	if (!Context->tagged)
 	{
-	  mutt_error _("No tagged messages.");
+          if (op == OP_TAG_PREFIX)
+            mutt_error _("No tagged messages.");
+          else if (op == OP_TAG_PREFIX_COND)
+          {
+            mutt_flush_macro_to_endcond ();
+            mutt_message  _("Nothing to do.");
+          }
 	  continue;
 	}
-	tag = 1;
 
-	/* give visual indication that the next command is a tag- command */
-	mutt_window_mvaddstr (MuttMessageWindow, 0, 0, "tag-");
-	mutt_window_clrtoeol (MuttMessageWindow);
-
-	/* get the real command */
-	if ((op = km_dokey (MENU_MAIN)) == OP_TAG_PREFIX)
-	{
-	  /* abort tag sequence */
-          mutt_window_clearline (MuttMessageWindow, 0);
-	  continue;
-	}
+        /* get the real command */
+        tag = 1;
+        continue;
       }
       else if (option (OPTAUTOTAG) && Context && Context->tagged)
 	tag = 1;
 
-      if (op == OP_TAG_PREFIX_COND)
-      {
-	if (!Context)
-	{
-	  mutt_error _("No mailbox is open.");
-	  continue;
-	}
-
-	if (!Context->tagged)
-	{
-	  mutt_flush_macro_to_endcond ();
-	  mutt_message  _("Nothing to do.");
-	  continue;
-	}
-	tag = 1;
-
-	/* give visual indication that the next command is a tag- command */
-	mutt_window_mvaddstr (MuttMessageWindow, 0, 0, "tag-");
-	mutt_window_clrtoeol (MuttMessageWindow);
-
-	/* get the real command */
-	if ((op = km_dokey (MENU_MAIN)) == OP_TAG_PREFIX)
-	{
-	  /* abort tag sequence */
-	  mutt_window_clearline (MuttMessageWindow, 0);
-	  continue;
-	}
-      }
-
       mutt_clear_error ();
     }
     else
@@ -895,7 +896,7 @@ int mutt_index_menu (void)
 
 	CHECK_ATTACH;
 	mutt_pattern_func (MUTT_DELETE, _("Delete messages matching: "));
-	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	break;
 
 #ifdef USE_POP
@@ -947,7 +948,6 @@ int mutt_index_menu (void)
 	  }
 	  else
 	    menu->current = 0;
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	  if (Context->msgcount && (Sort & SORT_MASK) == SORT_THREADS)
 	    mutt_draw_tree (Context);
 	  menu->redraw = REDRAW_FULL;
@@ -1030,7 +1030,7 @@ int mutt_index_menu (void)
 	{
 	  for (j = 0; j < Context->vcount; j++)
 	    mutt_set_flag (Context, Context->hdrs[Context->v2r[j]], MUTT_TAG, 0);
-	  menu->redraw = REDRAW_STATUS | REDRAW_INDEX;
+	  menu->redraw |= REDRAW_STATUS | REDRAW_INDEX;
 	}
 	else
 	{
@@ -1040,7 +1040,7 @@ int mutt_index_menu (void)
 	    ((Context->last_tag == CURHDR && !CURHDR->tagged)
 	     ? NULL : Context->last_tag);
 
-	  menu->redraw = REDRAW_STATUS;
+	  menu->redraw |= REDRAW_STATUS;
 	  if (option (OPTRESOLVE) && menu->current < Context->vcount - 1)
 	  {
 	    menu->current++;
@@ -1056,7 +1056,7 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	mutt_pattern_func (MUTT_TAG, _("Tag messages matching: "));
-	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	break;
 
       case OP_MAIN_UNDELETE_PATTERN:
@@ -1068,7 +1068,7 @@ int mutt_index_menu (void)
 	CHECK_ACL(MUTT_ACL_DELETE, _("Cannot undelete message(s)"));
 
 	if (mutt_pattern_func (MUTT_UNDELETE, _("Undelete messages matching: ")) == 0)
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	break;
 
       case OP_MAIN_UNTAG_PATTERN:
@@ -1076,7 +1076,7 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	if (mutt_pattern_func (MUTT_UNTAG, _("Untag messages matching: ")) == 0)
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	break;
 
 	/* --------------------------------------------------------------------
@@ -1215,7 +1215,7 @@ int mutt_index_menu (void)
 	{
 	  mutt_buffy (buf, sizeof (buf));
 
-          if (mutt_enter_fname (cp, buf, sizeof (buf), &menu->redraw, 1) == -1)
+          if (mutt_enter_fname (cp, buf, sizeof (buf), 1) == -1)
           {
             if (menu->menu == MENU_PAGER)
             {
@@ -1263,7 +1263,7 @@ int mutt_index_menu (void)
 
             FREE (&new_last_folder);
 	    set_option (OPTSEARCHINVALID);
-	    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	    menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	    break;
 	  }
 	  FREE (&Context);
@@ -1274,12 +1274,14 @@ int mutt_index_menu (void)
 
         mutt_sleep (0);
 
-	/* Set CurrentMenu to MENU_MAIN before executing any folder
-	 * hooks so that all the index menu functions are available to
-	 * the exec command.
-	 */
-
-	CurrentMenu = MENU_MAIN;
+        /* Note that menu->menu may be MENU_PAGER if the change folder
+         * operation originated from the pager.
+         *
+         * However, exec commands currently use CurrentMenu to determine what
+         * functions are available, which is automatically set by the
+         * mutt_push/pop_current_menu() functions.  If that changes, the menu
+         * would need to be reset here, and the pager cleanup code after the
+         * switch statement would need to be run. */
 	mutt_folder_hook (buf);
 
 	if ((Context = mx_open_mailbox (buf,
@@ -1326,12 +1328,21 @@ int mutt_index_menu (void)
 
 	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED)))
 	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
-	if ((op = mutt_display_message (CURHDR)) == -1)
+
+        /* If we are returning to the pager via an index menu redirection, we
+         * need to reset the menu->menu.  Otherwise mutt_pop_current_menu() will
+         * set CurrentMenu incorrectly when we return back to the index menu. */
+        menu->menu = MENU_MAIN;
+
+        if ((op = mutt_display_message (CURHDR)) < 0)
 	{
 	  unset_option (OPTNEEDRESORT);
 	  break;
 	}
 
+        /* This is used to redirect a single operation back here afterwards.  If
+         * mutt_display_message() returns 0, then the menu and pager state will
+         * be cleaned up after this switch statement. */
 	menu->menu = MENU_PAGER;
  	menu->oldcurrent = menu->current;
 	continue;
@@ -1553,11 +1564,11 @@ int mutt_index_menu (void)
 			       (op == OP_SAVE) || (op == OP_DECODE_SAVE),
 			       (op == OP_DECODE_SAVE) || (op == OP_DECODE_COPY),
 			       (op == OP_DECRYPT_SAVE) || (op == OP_DECRYPT_COPY) ||
-			       0,
-			       &menu->redraw) == 0 &&
+			       0) == 0 &&
 	     (op == OP_SAVE || op == OP_DECODE_SAVE || op == OP_DECRYPT_SAVE)
 	    )
 	{
+          menu->redraw |= REDRAW_STATUS;
 	  if (tag)
 	    menu->redraw |= REDRAW_INDEX;
 	  else if (option (OPTRESOLVE))
@@ -1700,13 +1711,13 @@ int mutt_index_menu (void)
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	    {
 	      menu->current = menu->oldcurrent;
-	      menu->redraw = REDRAW_CURRENT;
+	      menu->redraw |= REDRAW_CURRENT;
 	    }
 	    else
-	      menu->redraw = REDRAW_MOTION_RESYNCH;
+	      menu->redraw |= REDRAW_MOTION_RESYNCH;
 	  }
 	  else
-	    menu->redraw = REDRAW_CURRENT;
+	    menu->redraw |= REDRAW_CURRENT;
 	}
 	menu->redraw |= REDRAW_STATUS;
 	break;
@@ -1732,7 +1743,7 @@ int mutt_index_menu (void)
 		mutt_set_flag (Context, Context->hdrs[Context->v2r[j]], MUTT_READ, 1);
 	    }
 	  }
-	  menu->redraw = REDRAW_STATUS | REDRAW_INDEX;
+	  menu->redraw |= REDRAW_STATUS | REDRAW_INDEX;
 	}
 	else
 	{
@@ -1746,13 +1757,13 @@ int mutt_index_menu (void)
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	    {
 	      menu->current = menu->oldcurrent;
-	      menu->redraw = REDRAW_CURRENT;
+	      menu->redraw |= REDRAW_CURRENT;
 	    }
 	    else
-	      menu->redraw = REDRAW_MOTION_RESYNCH;
+	      menu->redraw |= REDRAW_MOTION_RESYNCH;
 	  }
 	  else
-	    menu->redraw = REDRAW_CURRENT;
+	    menu->redraw |= REDRAW_CURRENT;
 	  menu->redraw |= REDRAW_STATUS;
 	}
 	break;
@@ -1837,7 +1848,7 @@ int mutt_index_menu (void)
 
 	if (mutt_change_flag (tag ? NULL : CURHDR, (op == OP_MAIN_SET_FLAG)) == 0)
 	{
-	  menu->redraw = REDRAW_STATUS;
+	  menu->redraw |= REDRAW_STATUS;
 	  if (tag)
 	    menu->redraw |= REDRAW_INDEX;
 	  else if (option (OPTRESOLVE))
@@ -1952,20 +1963,18 @@ int mutt_index_menu (void)
 	CHECK_ATTACH;
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
-	ci_bounce_message (tag ? NULL : CURHDR, &menu->redraw);
+	ci_bounce_message (tag ? NULL : CURHDR);
 	break;
 
       case OP_CREATE_ALIAS:
 
         mutt_create_alias (Context && Context->vcount ? CURHDR->env : NULL, NULL);
-	MAYBE_REDRAW (menu->redraw);
         menu->redraw |= REDRAW_CURRENT;
 	break;
 
       case OP_QUERY:
 	CHECK_ATTACH;
 	mutt_query_menu (NULL, 0);
-	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_PURGE_MESSAGE:
@@ -1983,7 +1992,7 @@ int mutt_index_menu (void)
           mutt_tag_set_flag (MUTT_PURGE, (op == OP_PURGE_MESSAGE));
 	  if (option (OPTDELETEUNTAG))
 	    mutt_tag_set_flag (MUTT_TAG, 0);
-	  menu->redraw = REDRAW_INDEX;
+	  menu->redraw |= REDRAW_INDEX;
 	}
 	else
 	{
@@ -1996,7 +2005,7 @@ int mutt_index_menu (void)
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	    {
 	      menu->current = menu->oldcurrent;
-	      menu->redraw = REDRAW_CURRENT;
+	      menu->redraw |= REDRAW_CURRENT;
 	    }
 	    else if (menu->menu == MENU_PAGER)
 	    {
@@ -2007,7 +2016,7 @@ int mutt_index_menu (void)
 	      menu->redraw |= REDRAW_MOTION_RESYNCH;
 	  }
 	  else
-	    menu->redraw = REDRAW_CURRENT;
+	    menu->redraw |= REDRAW_CURRENT;
 	}
 	menu->redraw |= REDRAW_STATUS;
 	break;
@@ -2032,7 +2041,7 @@ int mutt_index_menu (void)
 	  if (option (OPTRESOLVE))
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	      menu->current = menu->oldcurrent;
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	}
 	break;
 
@@ -2045,13 +2054,8 @@ int mutt_index_menu (void)
 
       case OP_ENTER_COMMAND:
 
-	CurrentMenu = MENU_MAIN;
 	mutt_enter_command ();
 	mutt_check_rescore (Context);
-	if (option (OPTFORCEREDRAWINDEX))
-	  menu->redraw = REDRAW_FULL;
-	unset_option (OPTFORCEREDRAWINDEX);
-	unset_option (OPTFORCEREDRAWPAGER);
 	break;
 
       case OP_EDIT_MESSAGE:
@@ -2181,11 +2185,10 @@ int mutt_index_menu (void)
 	 */
 	if (Context->magic == MUTT_IMAP && !option (OPTIMAPPEEK))
 	{
-	  menu->redraw = (tag ? REDRAW_INDEX : REDRAW_CURRENT) | REDRAW_STATUS;
+	  menu->redraw |= (tag ? REDRAW_INDEX : REDRAW_CURRENT) | REDRAW_STATUS;
 	}
 #endif
 
-	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_PRINT:
@@ -2200,7 +2203,7 @@ int mutt_index_menu (void)
 	 */
 	if (Context->magic == MUTT_IMAP && !option (OPTIMAPPEEK))
 	{
-	  menu->redraw = (tag ? REDRAW_INDEX : REDRAW_CURRENT) | REDRAW_STATUS;
+	  menu->redraw |= (tag ? REDRAW_INDEX : REDRAW_CURRENT) | REDRAW_STATUS;
 	}
 #endif
 
@@ -2231,7 +2234,7 @@ int mutt_index_menu (void)
 	      continue;
 	    }
 	  }
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	}
 	break;
 
@@ -2314,7 +2317,6 @@ int mutt_index_menu (void)
       case OP_SHELL_ESCAPE:
 
 	mutt_shell_escape ();
-	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_TAG_THREAD:
@@ -2337,7 +2339,7 @@ int mutt_index_menu (void)
 	    if (menu->current == -1)
 	      menu->current = menu->oldcurrent;
 	  }
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	}
 	break;
 
@@ -2353,7 +2355,7 @@ int mutt_index_menu (void)
 	{
 	  mutt_tag_set_flag (MUTT_DELETE, 0);
 	  mutt_tag_set_flag (MUTT_PURGE, 0);
-	  menu->redraw = REDRAW_INDEX;
+	  menu->redraw |= REDRAW_INDEX;
 	}
 	else
 	{
@@ -2362,10 +2364,10 @@ int mutt_index_menu (void)
 	  if (option (OPTRESOLVE) && menu->current < Context->vcount - 1)
 	  {
 	    menu->current++;
-	    menu->redraw = REDRAW_MOTION_RESYNCH;
+	    menu->redraw |= REDRAW_MOTION_RESYNCH;
 	  }
 	  else
-	    menu->redraw = REDRAW_CURRENT;
+	    menu->redraw |= REDRAW_CURRENT;
 	}
 	menu->redraw |= REDRAW_STATUS;
 	break;
@@ -2396,7 +2398,7 @@ int mutt_index_menu (void)
 	    if (menu->current == -1)
 	      menu->current = menu->oldcurrent;
 	  }
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	  menu->redraw |= REDRAW_INDEX | REDRAW_STATUS;
 	}
 	break;
 
@@ -2437,7 +2439,6 @@ int mutt_index_menu (void)
       case OP_SIDEBAR_TOGGLE_VISIBLE:
 	toggle_option (OPTSIDEBAR);
         mutt_reflow_windows();
-	menu->redraw = REDRAW_FULL;
 	break;
 #endif
       default:
@@ -2450,14 +2451,12 @@ int mutt_index_menu (void)
       mutt_clear_pager_position ();
       menu->menu = MENU_MAIN;
       menu->redraw = REDRAW_FULL;
-#if 0
-      set_option (OPTWEED); /* turn header weeding back on. */
-#endif
     }
 
     if (done) break;
   }
 
+  mutt_pop_current_menu (menu);
   mutt_menuDestroy (&menu);
   return (close);
 }
