@@ -191,6 +191,8 @@ char *safe_strdup (const char *);
 int mutt_atos (const char *, short *);
 int mutt_atoi (const char *, int *);
 int mutt_atol (const char *, long *);
+int mutt_atoui (const char *, unsigned int *);
+int mutt_atoul (const char *, unsigned long *);
 
 const char *mutt_stristr (const char *, const char *);
 const char *mutt_basename (const char *);
