@@ -35,11 +35,13 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <stdlib.h>
+#include <signal.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#include <time.h>
 #include <sys/socket.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -152,13 +154,13 @@ int mutt_socket_write_d (CONNECTION *conn, const char *buf, int len, int dbg)
  *   Returns: >0 if there is data to read,
  *            0 if a read would block,
  *            -1 if this connection doesn't support polling */
-int mutt_socket_poll (CONNECTION* conn)
+int mutt_socket_poll (CONNECTION* conn, time_t wait_secs)
 {
   if (conn->bufpos < conn->available)
     return conn->available - conn->bufpos;
 
   if (conn->conn_poll)
-    return conn->conn_poll (conn);
+    return conn->conn_poll (conn, wait_secs);
 
   return -1;
 }
@@ -344,6 +346,7 @@ static int socket_connect (int fd, struct sockaddr* sa)
 {
   int sa_size;
   int save_errno;
+  sigset_t set;
 
   if (sa->sa_family == AF_INET)
     sa_size = sizeof (struct sockaddr_in);
@@ -356,12 +359,18 @@ static int socket_connect (int fd, struct sockaddr* sa)
     dprint (1, (debugfile, "Unknown address family!\n"));
     return -1;
   }
-  
+
   if (ConnectTimeout > 0)
       alarm (ConnectTimeout);
 
   mutt_allow_interrupt (1);
 
+  /* FreeBSD's connect() does not respect SA_RESTART, meaning
+   * a SIGWINCH will cause the connect to fail. */
+  sigemptyset (&set);
+  sigaddset (&set, SIGWINCH);
+  sigprocmask (SIG_BLOCK, &set, NULL);
+
   save_errno = 0;
 
   if (connect (fd, sa, sa_size) < 0)
@@ -374,6 +383,7 @@ static int socket_connect (int fd, struct sockaddr* sa)
   if (ConnectTimeout > 0)
       alarm (0);
   mutt_allow_interrupt (0);
+  sigprocmask (SIG_UNBLOCK, &set, NULL);
 
   return save_errno;
 }
@@ -422,18 +432,43 @@ int raw_socket_write (CONNECTION* conn, const char* buf, size_t count)
   return rc;
 }
 
-int raw_socket_poll (CONNECTION* conn)
+int raw_socket_poll (CONNECTION* conn, time_t wait_secs)
 {
   fd_set rfds;
-  struct timeval tv = { 0, 0 };
+  unsigned long wait_millis, post_t_millis;
+  struct timeval tv, pre_t, post_t;
+  int rv;
 
   if (conn->fd < 0)
     return -1;
 
-  FD_ZERO (&rfds);
-  FD_SET (conn->fd, &rfds);
-  
-  return select (conn->fd + 1, &rfds, NULL, NULL, &tv);
+  wait_millis = wait_secs * 1000UL;
+
+  FOREVER
+  {
+    tv.tv_sec = wait_millis / 1000;
+    tv.tv_usec = (wait_millis % 1000) * 1000;
+
+    FD_ZERO (&rfds);
+    FD_SET (conn->fd, &rfds);
+
+    gettimeofday (&pre_t, NULL);
+    rv = select (conn->fd + 1, &rfds, NULL, NULL, &tv);
+    gettimeofday (&post_t, NULL);
+
+    if (rv > 0 ||
+        (rv < 0 && errno != EINTR))
+      return rv;
+
+    if (SigInt)
+      mutt_query_exit ();
+
+    wait_millis += (pre_t.tv_sec * 1000UL) + (pre_t.tv_usec / 1000);
+    post_t_millis = (post_t.tv_sec * 1000UL) + (post_t.tv_usec / 1000);
+    if (wait_millis <= post_t_millis)
+      return 0;
+    wait_millis -= post_t_millis;
+  }
 }
 
 int raw_socket_open (CONNECTION* conn)
