@@ -647,7 +647,8 @@ static int tls_check_stored_hostname (const gnutls_datum_t *cert,
     if (REGCOMP(&preg, "^#H ([a-zA-Z0-9_\\.-]+) ([0-9A-F]{4}( [0-9A-F]{4}){7})[ \t]*$",
                 REG_ICASE) != 0)
     {
-       return 0;
+      safe_fclose (&fp);
+      return 0;
     }
 
     buf[0] = '\0';
