@@ -615,7 +615,7 @@ out:
   return rc;
 }
 
-static int pgp_check_traditional_one_body (FILE *fp, BODY *b, int tagged_only)
+static int pgp_check_traditional_one_body (FILE *fp, BODY *b)
 {
   char tempfile[_POSIX_PATH_MAX];
   char buf[HUGE_STRING];
@@ -628,9 +628,6 @@ static int pgp_check_traditional_one_body (FILE *fp, BODY *b, int tagged_only)
   if (b->type != TYPETEXT)
     return 0;
 
-  if (tagged_only && !b->tagged)
-    return 0;
-
   mutt_mktemp (tempfile, sizeof (tempfile));
   if (mutt_decode_save_attachment (fp, b, tempfile, 0, 0) != 0)
   {
@@ -675,21 +672,24 @@ static int pgp_check_traditional_one_body (FILE *fp, BODY *b, int tagged_only)
   return 1;
 }
 
-int pgp_check_traditional (FILE *fp, BODY *b, int tagged_only)
+int pgp_check_traditional (FILE *fp, BODY *b, int just_one)
 {
   int rv = 0;
   int r;
   for (; b; b = b->next)
   {
-    if (is_multipart (b))
-      rv = pgp_check_traditional (fp, b->parts, tagged_only) || rv;
+    if (!just_one && is_multipart (b))
+      rv = pgp_check_traditional (fp, b->parts, 0) || rv;
     else if (b->type == TYPETEXT)
     {
       if ((r = mutt_is_application_pgp (b)))
 	rv = rv || r;
       else
-	rv = pgp_check_traditional_one_body (fp, b, tagged_only) || rv;
+	rv = pgp_check_traditional_one_body (fp, b) || rv;
     }
+
+    if (just_one)
+      break;
   }
 
   return rv;
@@ -1701,7 +1701,7 @@ BODY *pgp_traditional_encryptsign (BODY *a, int flags, char *keylist)
   return b;
 }
 
-int pgp_send_menu (HEADER *msg, int *redraw)
+int pgp_send_menu (HEADER *msg)
 {
   pgp_key_t p;
   char input_signas[SHORT_STRING];
@@ -1829,7 +1829,6 @@ int pgp_send_menu (HEADER *msg, int *redraw)
 
         crypt_pgp_void_passphrase ();  /* probably need a different passphrase */
       }
-      *redraw = REDRAW_FULL;
       break;
 
     case 'b': /* (b)oth */
